/** Copyright (C) 2016, Florin Oprea <florinoprea.contact@gmail.com>
 **
 **  This program is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 **/


#ifndef BASICICONS_H
#define BASICICONS_H

#include <QIcon>

class BasicIcons
{
public:
    BasicIcons();
    QIcon basic256Icon;
    QIcon saveIcon;
    QIcon saveAsIcon;
    QIcon saveAllIcon;
    QIcon newIcon;
    QIcon printIcon;
    QIcon openIcon;
    QIcon cutIcon;
    QIcon copyIcon;
    QIcon pasteIcon;
    QIcon undoIcon;
    QIcon redoIcon;
    QIcon exitIcon;
    QIcon preferencesIcon;
    QIcon runIcon;
    QIcon debugIcon;
    QIcon stepIcon;
    QIcon breakIcon;
    QIcon stopIcon;
    QIcon fontIcon;
    QIcon helpIcon;
    QIcon webIcon;
    QIcon documentIcon;
    QIcon clearIcon;
    QIcon gridIcon;
    QIcon infoIcon;
    QIcon goNextIcon;
    QIcon goPreviousIcon;
    QIcon goUpIcon;
    QIcon goDownIcon;
    QIcon goHomeIcon;
    QIcon findIcon;
    QIcon zoomInIcon;
    QIcon zoomOutIcon;
    QIcon closeIcon;

};

#endif // BASICICONS_H
