/**	Copyright (C) 2006, Ian Paul Larsen.
 **
 **  This program is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 **/


#ifndef __VIEWWIDGETIFACE_H
#define __VIEWWIDGETIFACE_H

#include <qglobal.h>

#include <QtWidgets/QMenu>
#include <QtWidgets/QToolBar>

class ViewWidgetIFace
{
public:
  	ViewWidgetIFace();
	virtual ~ViewWidgetIFace();

	virtual bool initActions(QMenu *, QToolBar *);
	bool usesToolBar() { return m_usesToolBar; }
	bool usesMenu() { return m_usesMenu; }
	
protected:
	bool m_usesToolBar;
	bool m_usesMenu;
};

#endif
