#pragma once

// These type definitions are the data types stored in DataElement, the stack, and variables


// stack and variable type T_REF used to pass a variable reference

#define T_UNASSIGNED	0
#define T_INT			1
#define T_FLOAT			2
#define T_STRING		3
#define T_ARRAY			4
#define T_REF			5
#define T_MAP			6

